<nav class="navbar navbar-expand-lg bg-body-tertiary position-absolute start-50">
    {% if current_user.is_authenticated %}
    <a class="nav-link" href="{{url_for('logout')}}">Logout</a>
    {% else %}
    <a class="nav-link" href="{{url_for('login')}}">Login</a>
    {% endif %}
</nav>