{% extends 'base.html' %}

{% block content %}

{% if current_user.is_authenticated %}
    <h2>Hi {{ current_user.user_name }}!</h2>
    {% for unused_app in current_user.unused_applications %}
        <li><a href="/add_app/{{unused_app}}">add_{{unused_app}}_app</a></li>
    {% endfor %}

{% else %}
    Please log in or register:
    <a href="/register">register</a>
{% endif %}

{%endblock%}