{% extends 'base.html' %}

{% block content %}

<head>Register</head>

<form method="post" action="/register">
    User Name
    <input type="text" name="username" required/>
    Email
    <input type="email" name="email" required/>
    Password
    <input type="password" name="password" required/>
    <input type="submit"/>
</form>
<br>

{% endblock %}
