{% extends 'base.html' %}

{% block content %}

<head>Add to </head>

<form method="post" action="/add_cheltuieli">
    <div class="container text-center">
        <div class="row">
            <div class="col">
                <label>Name</label>
                <input type="text" name="name" required/>
            </div>
            <div class="col">
                <label>Value[€]</label>
                <input type="number" name="value" step="any" required/>
            </div>
            <div class="col">
                <label>MyConto</label>
                <select id="cars" name="myconto">
                    <option value="EC">EC</option>
                    <option value="Siri&Radu">Siri&Radu</option>
                    <option value="N26">N26</option>
                    <option value="DeutscheBank">DeutscheBank</option>
                    <option value="N26 Family Mircea">N26 Family Mircea</option>
                </select>
            </div>
            <div class="col">
                <label>pay_day</label>
                <input type="date" name="pay_day"/>
            </div>
            <div class="col">
                <input type="submit" name="add_one_time_chelt"/>
            </div>
        </div>
    </div>
</form>
<br>

{% endblock %}
