{% extends 'base.html' %}

{% block content %}

<html lang="en">
<head>
	<meta charset="UTF-8" />
	<meta http-equiv="X-UA-Compatible" content="IE=edge" />
	<meta name="viewport" content="width=device-width, initial-scale=1.0" />
	<title>Login</title>
	<style>
		h1{
		color: green;
		}
	</style>
</head>
<body>
	<h1>Login to your account</h1>
	<form action="#" method="post">
	<label for="username">Username:</label>
	<input type="text" name="username" />
	<label for="password">Password:</label>
	<input type="password" name="password" />
	<button type="submit">Submit</button>
	</form>
</body>
</html>

{% endblock %}