<!--#https://www.w3schools.com/howto/howto_js_sidenav.asp-->

<head>
<meta name="viewport" content="width=device-width, initial-scale=1">
<style>
body {
  font-family: "Lato", sans-serif;
}

.sidenav {
  height: 100%;
  width: 0;
  position: fixed;
  z-index: 1;
  top: 0;
  left: 0;
  background-color: #111;
  overflow-x: hidden;
  transition: 0.5s;
  padding-top: 60px;
}

.sidenav a {
  padding: 8px 8px 8px 32px;
  text-decoration: none;
  font-size: 25px;
  color: #818181;
  display: block;
  transition: 0.3s;
}

.sidenav a:hover {
  color: #f1f1f1;
}

.sidenav .closebtn {
  position: absolute;
  top: 0;
  right: 25px;
  font-size: 36px;
  margin-left: 50px;
}

@media screen and (max-height: 450px) {
  .sidenav {padding-top: 15px;}
  .sidenav a {font-size: 18px;}
}
</style>
</head>
<body>

<div id="mySidenav" class="sidenav">
  <a href="javascript:void(0)" class="closebtn" onclick="closeNav()">&times;</a>
  <div class="container-fluid">
  <a class="nav-link active" aria-current="page" href="{{url_for('index')}}">Home</a>
  </div>

  {% if current_user.is_authenticated %}

      <a class="nav-link" href="{{url_for('masina')}}">Masina</a>
      <a class="nav-link" href="{{url_for('cheltuieli')}}">Cheltuieli</a>
      <a class="nav-link" href="{{url_for('aeroclub')}}">Aeroclub</a>

  {% else %}
  {% endif %}

</div>

{% if current_user.is_authenticated %}
<span style="font-size:30px;cursor:pointer" onclick="openNav()">&#9776; {{ current_user.user_name }}</span>
{% else %}
{% endif %}
<script>
function openNav() {
  document.getElementById("mySidenav").style.width = "250px";
}

function closeNav() {
  document.getElementById("mySidenav").style.width = "0";
}
</script>

</body>
</html>
