{% extends 'base.html' %}
{% block content %}

<div class="container text-center" style="border: 1px solid black">
    <div class="row">
        <div class="col">
            Total Unpaid Flights: {{total_unpaid_flights}}
        </div>
    </div>
    <div class="row">
        <div class="col">
            Total Flight time this year: {{flight_time_this_year}}
        </div>
    </div>
    <div class="row">
        <div class="col">
            Total Flight Time: {{total_flight_time}}
        </div>
    </div>
</div>
<br>

{% endblock %}